
import os
import fleep

from .config import PATH_DELIMITER

class DownloadHandler:
    """
    docstring for RequestHandler.
    """

    def __init__(self, request_handler, target_dir):
        self.request_handler = request_handler
        self.target_dir = target_dir

    def _determine_extension(self, file_loc):
        with open(file_loc, "rb") as file:
            info = fleep.get(file.read(256))
        if not info.extension:
            # default extension
            extension = "txt"
        else:
            extension = info.extension[0]
        if extension == "ai":
            extension = "pdf"
        elif extension == "pages":
            extension = "zip"
        return extension

    def download(self, file_dict, file_path, ignore_endings):
        # print(f"download <{file_dict=}>, <{file_path=}>")
        path = ""
        print(f"{self.target_dir=}")
        if self.target_dir[-1] != PATH_DELIMITER:
            # print(self.target_dir[:-1])
            self.target_dir += PATH_DELIMITER

        # print(f"{self.target_dir=}")
        # relative path in script execution dir
        if self.target_dir == PATH_DELIMITER:
            path += os.getcwd()

        # else: an absolute path has been set and will be used
        path += self.target_dir + file_path

        # create path if does not exist
        self._verify_path(path)
        url = file_dict["url"]
        file_name = file_dict["file_name"]
        file = self.request_handler.get_file(url)
        file_loc = path+PATH_DELIMITER+file_name
        # save file without file-ending
        with open(file_loc, "wb") as file_we:
            file_we.write(file.content)
        # set file-ending
        extension = self._determine_extension(file_loc)
        # if extension in ignore_endings:
        #     # delete file
        os.rename(r""+file_loc,r""+file_loc+"."+extension)

    def _verify_path(self, path):
        folder_list = path.split(PATH_DELIMITER)
        for i in range(0, len(folder_list)):
            sub_path = PATH_DELIMITER+PATH_DELIMITER.join(folder_list[1:i+3])
            if not os.path.exists(sub_path):
                # todo check windows
                original_umask = os.umask(000)
                os.makedirs(sub_path, mode=0o777), #mode = 0o666)
                os.umask(original_umask)
        return 1




import platform

SCRAPER_NAME = "ilias-scraper"
LOGIN_POST_URL = "https://ilias.uni-konstanz.de/ilias/ilias.php?lang=de&client_id=ilias_uni&cmd=post&cmdClass=ilstartupgui&cmdNode=y1&baseClass=ilStartUpGUI&rtoken="


if platform.system() == "windows":
    PATH_DELIMITER = "\\"
else:
    PATH_DELIMITER = "/"




import os
import glob
from tqdm import tqdm
from colorama import Fore, Back, Style

from .config import PATH_DELIMITER
from .login_handler import LoginHandler
from .file_parser import FileParser
from .request_handler import RequestHandler
from .download_handler import DownloadHandler


class Controller:
    """
    The controller of the package - here comes everything together
    """

    def __init__(self, username, target_dir="/", skip_existing_files=True, ignore=[]):
        self.request_handler = RequestHandler()
        self.username = username
        self.target_dir = target_dir
        self.ignore_endings = ignore
        self._check_version()
        self.skip_existing_files = skip_existing_files
        if self.skip_existing_files:
            print("getting files")
            self.existing_files = self.get_existing_files()
        else:
            self.existing_files = []
        print(self.existing_files)


        self.login_handler = LoginHandler(self.username)

        self.test_course_url = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_crs_1078392.html"
        self.test_file1 = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_file_1078407_download.html"
        self.test_file2 = "https://ilias.uni-konstanz.de/ilias/goto_ilias_uni_file_1091580_download.html"

    def _check_version(self):
        version_path = os.path.join(os.path.dirname(__file__),"VERSION")
        with open(version_path) as f:
            line = f.readline()
            local_version = line.replace("\n", "")
        remote_version = self.request_handler.get_version()
        if remote_version != local_version:
            print(Fore.RED + Style.BRIGHT + Back.BLACK \
            + f"You are using version {local_version}, however version {remote_version} is available.\n" \
            + "THIS VERSION IS DEPRECATED AND POSSIBLY UNSTABLE.\n" \
            + "PLEASE UPDATE VIA:\n" \
            + Style.NORMAL + Fore.BLACK + Back.WHITE \
            + "pip install iliasScraper --upgrade" + Style.RESET_ALL \
            + "\n"+(50*"-")+"\n")

    def get_existing_files(self):
        # existing_files = []
        directory = self.target_dir
        if directory == PATH_DELIMITER:
            directory = os.getcwd()

        # filename.split(PATH_DELIMITER)[-1]
        existing_files = [os.path.join(dp, f) for dp, dn, filenames in os.walk(directory) for f in filenames]
        return existing_files


    def collect(self, url):
        file_dict = {}
        print(">> Getting all folder and session urls...")
        all_urls = self.file_parser.get_all_urls(url)#.append(url)
        all_urls.append(url)
        print(Style.RESET_ALL)
        for link in all_urls:
            soup = self.request_handler.get_soup(link)
            file_dict_for_link = self.file_parser.parse(soup)
            file_dict.update(file_dict_for_link)
        return file_dict

    def init_controller(self):
        self.session = self.login_handler.login()
        self.request_handler.init_session(self.session)
        self.file_parser = FileParser(self.request_handler, self.skip_existing_files, self.existing_files)
        self.download_handler = DownloadHandler(self.request_handler, self.target_dir)

    def download(self, url, name="course name"):
        print(Fore.BLUE + "\t\t ILIAS SCRAPER")
        print(Fore.GREEN + f" - {name} -")
        print(Style.RESET_ALL)
        files = self.collect(url)
        print(">> Starting download")
        for path in files.keys():
            files_for_path = files[path]
            print(Style.DIM + path)
            for file in files_for_path:
                print(file["file_name"])
                self.download_handler.download(file, path, self.ignore_endings)

    def test(self):
        self.download_handler.download({"url":self.test_file2, "file_name":"testfile"}, "")


# controller = Controller("tilman.kerl", "", ignore="None")
# controller.init_controller()
# controller.download(controller.test_course_url, "bsc")
# controller.test()




import re
from tqdm import tqdm
from colorama import Fore, Back, Style

class FileParser:
    """
    docstring for FileParser.
    """

    def __init__(self, request_handler, skip_existing_files=True, existing_files=[]):
        self.parser = 'html.parser'
        self.ignore = ["weblink", "test", "forum", "übung", "inhaltsseite"]
        self.file_id = "datei"
        self.folder_id = "ordner"
        self.current_path = "/"
        self.dir_dict = {}
        self.done_folders = []
        self.folder_todo = []
        self.todo_folders_index = 0
        self.layer = 0
        self.request_handler = request_handler
        self.all_urls = []
        self.skip_existing_files = skip_existing_files
        self.existing_files = existing_files


    def filter_ignored(self, img_list):
        def filter_(title):
            return any(map(lambda el: el in title.lower(), self.ignore))
        return [tag for tag in img_list if not filter_(tag["title"])]

    def _extract_href(self, img):
        element = list(img.parent.parent.find_all("a"))[0]
        return element["href"]

    def _pathify_folder_name(self, folder_name):
        folder_name = "_".join(folder_name.split(" "))
        folder_name = "_".join(folder_name.split("|"))
        folder_name = "_".join(folder_name.split("'"))
        folder_name = "_".join(folder_name.split(":"))
        folder_name = re.sub("_+", "_", folder_name)
        return folder_name

    def _extract_file_name(self, img):
        # why was index = 1 ????
        element = list(img.parent.parent.parent.find_all("a"))[0]
        return element.text

    def _extract_symbol_name(self, a_tag):
        imgs = a_tag.find_all("img")
        if imgs:
            return str(imgs[0]["title"].lower())
        return ""

    def update_dir_dict(self, img_list, path):
        """
        """
        dir_dict = {}
        for img in img_list:
            img_title = img["title"].lower()
            # print(f"{img_title=}")
            if self.file_id in img_title:
                if not path in dir_dict:
                    dir_dict[path] = []
                filename = self._extract_file_name(img)
                # print(f"{filename=}")
                if self.skip_existing_files and filename in self.existing_files:
                    print("continue")
                    continue
                dir_dict[path].append({
                        "url": self._extract_href(img),
                        "file_name": filename
                        })
        return dir_dict

    def _is_sitzung(self, tag, soup, add_q=False):
        if add_q:
            sitzungen = soup.find_all("img", title="Symbol Sitzung")
            sitzungen_no = len(sitzungen)
            if sitzungen_no == 1:
                return True
            return False

        img = tag.parent.parent.parent.parent.parent.find_all("img", class_="ilListItemIcon")
        if img:
            img = img[0]
            title = img["title"].lower()
            if "sitzung" in title or "session" in title:
                return True
        return False

    def get_all_urls(self, url):
        # print(40*"~")
        # print(f"{url=}")
        soup = self.request_handler.get_soup(url)
        links = soup.find_all('a', class_="il_ContainerItemTitle")
        if not links or url in self.all_urls:
            self.all_urls.append(url)
            return 1
        else:
            sitzung_q = self._is_sitzung(url, soup=soup, add_q=True)
            # print(f"{sitzung_q=}")
            if "fold" in url or sitzung_q:
                self.all_urls.append(url)
                print(Style.DIM + url + Style.RESET_ALL)
            for link in links:
                if not link.has_attr("href") or link in self.all_urls:
                    continue
                href = link['href']
                if not "https" in href:
                    href = f"https://ilias.uni-konstanz.de/ilias/{href}"

                if "fold" in href or self._is_sitzung(link, soup=soup):
                    self.get_all_urls(href)

        return list(set(self.all_urls))

    def extract_path_from_breadcrumb(self, breadcrumb_string):
        total_path_list = breadcrumb_string.split("\n")
        split_index = list(map(lambda item: "Lehrveranstaltung" in item, total_path_list)).index(True)
        path_list = total_path_list[split_index+1:][:-1]
        return path_list

    def parse(self, soup):
        breadcrumb_string = soup.find_all("ol", class_="breadcrumb")
        if not breadcrumb_string:
            return {}
        breadcrumb_string = breadcrumb_string[0].text
        path = "/".join(list(map(lambda x: self._pathify_folder_name(x),
                        self.extract_path_from_breadcrumb(breadcrumb_string))))

        # ilContainerListItemOuter
        # - ilContainerListItemIcon
        # - ilContainerListItemContent

        all_img_tags = soup.find_all("img", class_="ilListItemIcon")
        f_img_tags = self.filter_ignored(all_img_tags)

        dir_dict = self.update_dir_dict(f_img_tags, path)

        return dir_dict
